@@ -11,10 +11,9 @@
 # distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
 # ANY KIND, either express or implied. See the License for the specific
 # language governing permissions and limitations under the License.
-import sys
+import io
 import logging
 import functools
-import socket
 
 import urllib3.util
 from urllib3.connection import VerifiedHTTPSConnection
@@ -24,8 +23,10 @@ from urllib3.connectionpool import HTTPSConnectionPool
 
 import botocore.utils
 from botocore.compat import six
-from botocore.compat import HTTPHeaders, HTTPResponse, urlunsplit, urlsplit, \
-     urlencode, MutableMapping
+from botocore.compat import (
+    HTTPHeaders, HTTPResponse, urlunsplit, urlsplit,
+    urlencode, urlparse, MutableMapping
+)
 from botocore.exceptions import UnseekableStreamError
 
 
@@ -81,38 +82,6 @@ class AWSConnection(object):
         self._expect_header_set = False
         self.response_class = self._original_response_cls
 
-    def _tunnel(self):
-        # Works around a bug in py26 which is fixed in later versions of
-        # python. Bug involves hitting an infinite loop if readline() returns
-        # nothing as opposed to just ``\r\n``.
-        # As much as I don't like having if py2: <foo> code blocks, this seems
-        # the cleanest way to handle this workaround.  Fortunately, the
-        # difference from py26 to py3 is very minimal.  We're essentially
-        # just overriding the while loop.
-        if sys.version_info[:2] != (2, 6):
-            return super(AWSConnection, self)._tunnel()
-
-        # Otherwise we workaround the issue.
-        self._set_hostport(self._tunnel_host, self._tunnel_port)
-        self.send("CONNECT %s:%d HTTP/1.0\r\n" % (self.host, self.port))
-        for header, value in self._tunnel_headers.iteritems():
-            self.send("%s: %s\r\n" % (header, value))
-        self.send("\r\n")
-        response = self.response_class(self.sock, strict=self.strict,
-                                       method=self._method)
-        (version, code, message) = response._read_status()
-
-        if code != 200:
-            self.close()
-            raise socket.error("Tunnel connection failed: %d %s" %
-                               (code, message.strip()))
-        while True:
-            line = response.fp.readline()
-            if not line:
-                break
-            if line in (b'\r\n', b'\n', b''):
-                break
-
     def _send_request(self, method, url, body, headers, *args, **kwargs):
         self._response_received = False
         if headers.get('Expect', b'') == b'100-continue':
@@ -238,8 +207,10 @@ class AWSConnection(object):
         parts = maybe_status_line.split(None, 2)
         # Check for HTTP/<version> 100 Continue\r\n
         return (
-            len(parts) >= 3 and parts[0].startswith(b'HTTP/') and
-            parts[1] == b'100')
+            len(parts) >= 3
+            and parts[0].startswith(b'HTTP/')
+            and parts[1] == b'100'
+        )
 
 
 class AWSHTTPConnection(AWSConnection, HTTPConnection):
@@ -378,8 +349,10 @@ class AWSRequestPreparer(object):
     def _prepare_url(self, original):
         url = original.url
         if original.params:
+            url_parts = urlparse(url)
+            delim = '&' if url_parts.query else '?'
             params = urlencode(list(original.params.items()), doseq=True)
-            url = '%s?%s' % (url, params)
+            url = delim.join((url, params))
         return url
 
     def _prepare_headers(self, original, prepared_body=None):
@@ -431,17 +404,23 @@ class AWSRequestPreparer(object):
         # Try asking the body for it's length
         try:
             return len(body)
-        except (AttributeError, TypeError) as e:
+        except (AttributeError, TypeError):
             pass
 
         # Try getting the length from a seekable stream
         if hasattr(body, 'seek') and hasattr(body, 'tell'):
-            orig_pos = body.tell()
-            body.seek(0, 2)
-            end_file_pos = body.tell()
-            body.seek(orig_pos)
-            return end_file_pos - orig_pos
-
+            try:
+                orig_pos = body.tell()
+                body.seek(0, 2)
+                end_file_pos = body.tell()
+                body.seek(orig_pos)
+                return end_file_pos - orig_pos
+            except io.UnsupportedOperation:
+                # in case when body is, for example, io.BufferedIOBase object
+                # it has "seek" method which throws "UnsupportedOperation"
+                # exception in such case we want to fall back to "chunked"
+                # encoding
+                pass
         # Failed to determine the length
         return None
 
