@@ -65,6 +65,17 @@ class UnknownServiceError(DataNotFoundError):
         "{known_service_names}")
 
 
+class UnknownRegionError(BotoCoreError):
+    """Raised when trying to load data for an unknown region.
+
+    :ivar region_name: The name of the unknown region.
+
+    """
+    fmt = (
+        "Unknown region: '{region_name}'. {error_msg}"
+    )
+
+
 class ApiVersionNotFoundError(BotoCoreError):
     """
     The data associated with either the API version or a compatible one
@@ -78,6 +89,7 @@ class ApiVersionNotFoundError(BotoCoreError):
 
 class HTTPClientError(BotoCoreError):
     fmt = 'An HTTP Client raised an unhandled exception: {error}'
+
     def __init__(self, request=None, response=None, **kwargs):
         self.request = request
         self.response = response
@@ -97,8 +109,10 @@ class InvalidIMDSEndpointError(BotoCoreError):
 
 
 class InvalidIMDSEndpointModeError(BotoCoreError):
-    fmt = ('Invalid EC2 Instance Metadata endpoint mode: {mode}'
-        ' Valid endpoint modes (case-insensitive): {valid_modes}.')
+    fmt = (
+        'Invalid EC2 Instance Metadata endpoint mode: {mode}'
+        ' Valid endpoint modes (case-insensitive): {valid_modes}.'
+    )
 
 
 class EndpointConnectionError(ConnectionError):
@@ -191,6 +205,19 @@ class NoRegionError(BaseEndpointResolverError):
     fmt = 'You must specify a region.'
 
 
+class EndpointVariantError(BaseEndpointResolverError):
+    """
+    Could not construct modeled endpoint variant.
+
+    :ivar error_msg: The message explaining why the modeled endpoint variant
+        is unable to be constructed.
+
+    """
+
+    fmt = ('Unable to construct a modeled endpoint with the following '
+           'variant(s) {tags}: ')
+
+
 class UnknownEndpointError(BaseEndpointResolverError, ValueError):
     """
     Could not construct an endpoint.
@@ -621,6 +648,10 @@ class MD5UnavailableError(BotoCoreError):
     fmt = "This system does not support MD5 generation."
 
 
+class MissingDependencyException(BotoCoreError):
+    fmt = "Missing Dependency: {msg}"
+
+
 class MetadataRetrievalError(BotoCoreError):
     fmt = "Error retrieving metadata: {error_msg}"
 
