@@ -23,7 +23,6 @@ import random
 import os
 import socket
 import cgi
-import warnings
 
 import dateutil.parser
 from dateutil.tz import tzutc
@@ -32,8 +31,9 @@ import botocore
 import botocore.awsrequest
 import botocore.httpsession
 from botocore.compat import (
-        json, quote, zip_longest, urlsplit, urlunsplit, OrderedDict,
-        six, urlparse, get_tzinfo_options, get_md5, MD5_AVAILABLE
+    json, quote, zip_longest, urlsplit, urlunsplit, OrderedDict,
+    six, urlparse, get_tzinfo_options, get_md5, MD5_AVAILABLE,
+    HAS_CRT
 )
 from botocore.vendored.six.moves.urllib.request import getproxies, proxy_bypass
 from botocore.exceptions import (
@@ -44,7 +44,7 @@ from botocore.exceptions import (
     InvalidRegionError, InvalidIMDSEndpointError, InvalidIMDSEndpointModeError,
     UnsupportedOutpostResourceError, UnsupportedS3ControlConfigurationError,
     UnsupportedS3ControlArnError, InvalidHostLabelError, HTTPClientError,
-    UnsupportedS3ConfigurationError,
+    UnsupportedS3ConfigurationError, MissingDependencyException
 )
 from urllib3.exceptions import LocationParseError
 
@@ -186,8 +186,10 @@ def ensure_boolean(val):
     """
     if isinstance(val, bool):
         return val
-    else:
+    elif isinstance(val, str):
         return val.lower() == 'true'
+    else:
+        return False
 
 
 def resolve_imds_endpoint_mode(session):
@@ -207,7 +209,7 @@ def resolve_imds_endpoint_mode(session):
             raise InvalidIMDSEndpointModeError(**error_msg_kwargs)
         return lendpoint_mode
     elif session.get_config_variable('imds_use_ipv6'):
-            return 'ipv6'
+        return 'ipv6'
     return 'ipv4'
 
 
@@ -220,10 +222,24 @@ def is_json_value_header(shape):
     :return: True if this type is a jsonvalue, False otherwise
     :rtype: Bool
     """
-    return (hasattr(shape, 'serialization') and
-            shape.serialization.get('jsonvalue', False) and
-            shape.serialization.get('location') == 'header' and
-            shape.type_name == 'string')
+    return (
+        hasattr(shape, 'serialization') and
+        shape.serialization.get('jsonvalue', False) and
+        shape.serialization.get('location') == 'header' and
+        shape.type_name == 'string'
+    )
+
+
+def has_header(header_name, headers):
+    """Case-insensitive check for header key."""
+    if header_name is None:
+        return False
+    elif isinstance(headers, botocore.awsrequest.HeadersDict):
+        return header_name in headers
+    else:
+        return header_name.lower() in [
+            key.lower() for key in headers.keys()
+        ]
 
 
 def get_service_module_name(service_model):
@@ -327,6 +343,13 @@ def set_value_from_jmespath(source, expression, value, is_first=True):
     source[current_key] = value
 
 
+def is_global_accesspoint(context):
+    """Determine if request is intended for an MRAP accesspoint."""
+    s3_accesspoint = context.get('s3_accesspoint', {})
+    is_global = s3_accesspoint.get('region') == ''
+    return is_global
+
+
 class _RetriesExceededError(Exception):
     """Internal exception used when the number of retries are exceeded."""
     pass
@@ -372,7 +395,9 @@ class IMDSFetcher(object):
         custom_metadata_endpoint = config.get('ec2_metadata_service_endpoint')
 
         if requires_ipv6 and custom_metadata_endpoint:
-            logger.warn("Custom endpoint and IMDS_USE_IPV6 are both set. Using custom endpoint.")
+            logger.warning(
+                "Custom endpoint and IMDS_USE_IPV6 are both set. Using custom endpoint."
+            )
 
         chosen_base_url = None
 
@@ -1010,6 +1035,7 @@ def is_valid_ipv6_endpoint_url(endpoint_url):
     netloc = urlparse(endpoint_url).netloc
     return IPV6_ADDRZ_RE.match(netloc) is not None
 
+
 def is_valid_endpoint_url(endpoint_url):
     """Verify the endpoint_url is valid.
 
@@ -1037,9 +1063,11 @@ def is_valid_endpoint_url(endpoint_url):
         re.IGNORECASE)
     return allowed.match(hostname)
 
+
 def is_valid_uri(endpoint_url):
     return is_valid_endpoint_url(endpoint_url) or is_valid_ipv6_endpoint_url(endpoint_url)
 
+
 def validate_region_name(region_name):
     """Provided region_name must be a valid host label."""
     if region_name is None:
@@ -1210,7 +1238,7 @@ def switch_host_s3_accelerate(request, operation_name, **kwargs):
 
     if operation_name in ['ListBuckets', 'CreateBucket', 'DeleteBucket']:
         return
-    _switch_hosts(request, endpoint,  use_new_scheme=False)
+    _switch_hosts(request, endpoint, use_new_scheme=False)
 
 
 def switch_host_with_param(request, param_name):
@@ -1545,10 +1573,13 @@ class S3EndpointSetter(object):
     _DEFAULT_DNS_SUFFIX = 'amazonaws.com'
 
     def __init__(self, endpoint_resolver, region=None,
-                 s3_config=None, endpoint_url=None, partition=None):
+                 s3_config=None, endpoint_url=None, partition=None,
+                 use_fips_endpoint=False):
+        # This is calling the endpoint_resolver in regions.py
         self._endpoint_resolver = endpoint_resolver
         self._region = region
         self._s3_config = s3_config
+        self._use_fips_endpoint = use_fips_endpoint
         if s3_config is None:
             self._s3_config = {}
         self._endpoint_url = endpoint_url
@@ -1558,6 +1589,7 @@ class S3EndpointSetter(object):
 
     def register(self, event_emitter):
         event_emitter.register('before-sign.s3', self.set_endpoint)
+        event_emitter.register('choose-signer.s3', self.set_signer)
         event_emitter.register(
             'before-call.s3.WriteGetObjectResponse',
             self.update_endpoint_to_s3_object_lambda
@@ -1575,10 +1607,12 @@ class S3EndpointSetter(object):
             return
 
         resolver = self._endpoint_resolver
+        # Constructing endpoints as s3-object-lambda as region
         resolved = resolver.construct_endpoint('s3-object-lambda', self._region)
 
         # Ideally we would be able to replace the endpoint before
         # serialization but there's no event to do that currently
+        # host_prefix is all the arn/bucket specs
         new_endpoint = 'https://{host_prefix}{hostname}'.format(
             host_prefix=params['host_prefix'],
             hostname=resolved['hostname'],
@@ -1598,6 +1632,14 @@ class S3EndpointSetter(object):
             self._switch_to_accesspoint_endpoint(request, region_name)
             return
         if self._use_accelerate_endpoint:
+            if self._use_fips_endpoint:
+                raise UnsupportedS3ConfigurationError(
+                    msg=(
+                        'Client is configured to use the FIPS psuedo region '
+                        'for "%s", but S3 Accelerate does not have any FIPS '
+                        'compatible endpoints.' % (self._region)
+                    )
+                )
             switch_host_s3_accelerate(request=request, **kwargs)
         if self._s3_addressing_handler:
             self._s3_addressing_handler(request=request, **kwargs)
@@ -1606,8 +1648,14 @@ class S3EndpointSetter(object):
         return 's3_accesspoint' in request.context
 
     def _validate_fips_supported(self, request):
-        if 'fips' not in self._region:
+        if not self._use_fips_endpoint:
             return
+        if 'fips' in request.context['s3_accesspoint']['region']:
+            raise UnsupportedS3AccesspointConfigurationError(
+                msg={
+                    'Invalid ARN, FIPS region not allowed in ARN.'
+                }
+            )
         if 'outpost_name' in request.context['s3_accesspoint']:
             raise UnsupportedS3AccesspointConfigurationError(
                 msg=(
@@ -1616,28 +1664,20 @@ class S3EndpointSetter(object):
                         self._region)
                 )
             )
-        client_region = self._region.replace('fips-', '').replace('-fips', '')
+        # Transforming psuedo region to actual region
         accesspoint_region = request.context['s3_accesspoint']['region']
-        if accesspoint_region != client_region:
-            if self._s3_config.get('use_arn_region', True):
+        if accesspoint_region != self._region:
+            if not self._s3_config.get('use_arn_region', True):
+                # TODO: Update message to reflect use_arn_region
+                # is not set
                 raise UnsupportedS3AccesspointConfigurationError(
                     msg=(
                         'Client is configured to use the FIPS psuedo-region '
-                        '"%s", but the access-point ARN provided is for the '
-                        '"%s" region. The use_arn_region configuration does '
-                        'not allow for cross-region calls when a FIPS '
-                        'pseudo-region is configured.' % (
-                            self._region, accesspoint_region)
-                    )
-                )
-            else:
-                raise UnsupportedS3AccesspointConfigurationError(
-                    msg=(
-                        'Client is configured to use the FIPS psuedo-region '
-                        '"%s", but the access-point ARN provided is for the '
-                        '"%s" region. For clients using a FIPS psuedo-region '
-                        'calls to access-point ARNs in another region are not '
-                        'allowed.' % (self._region, accesspoint_region)
+                        'for "%s", but the access-point ARN provided is for '
+                        'the "%s" region. For clients using a FIPS '
+                        'psuedo-region calls to access-point ARNs in another '
+                        'region are not allowed.' % (self._region,
+                                                     accesspoint_region)
                     )
                 )
 
@@ -1687,9 +1727,31 @@ class S3EndpointSetter(object):
                     'when an outpost ARN is specified.'
                 )
             )
+        self._validate_mrap_s3_config(request)
+
+    def _validate_mrap_s3_config(self, request):
+        if not is_global_accesspoint(request.context):
+            return
+        if self._s3_config.get('s3_disable_multiregion_access_points'):
+            raise UnsupportedS3AccesspointConfigurationError(
+                msg=(
+                    'Invalid configuration, Multi-Region Access Point '
+                    'ARNs are disabled.'
+                )
+            )
+        elif self._s3_config.get('use_dualstack_endpoint'):
+            raise UnsupportedS3AccesspointConfigurationError(
+                msg=(
+                    'Client does not support s3 dualstack configuration '
+                    'when a Multi-Region Access Point ARN is specified.'
+                )
+            )
 
     def _resolve_region_for_accesspoint_endpoint(self, request):
-        if self._s3_config.get('use_arn_region', True):
+        if is_global_accesspoint(request.context):
+            # Requests going to MRAP endpoints MUST be set to any (*) region.
+            self._override_signing_region(request, '*')
+        elif self._s3_config.get('use_arn_region', True):
             accesspoint_region = request.context['s3_accesspoint']['region']
             # If we are using the region from the access point,
             # we will also want to make sure that we set it as the
@@ -1698,6 +1760,17 @@ class S3EndpointSetter(object):
             return accesspoint_region
         return self._region
 
+    def set_signer(self, context, **kwargs):
+        if is_global_accesspoint(context):
+            if HAS_CRT:
+                return 's3v4a'
+            else:
+                raise MissingDependencyException(
+                    msg="Using S3 with an MRAP arn requires an additional "
+                    "dependency. You will need to pip install "
+                    "botocore[crt] before proceeding."
+                )
+
     def _resolve_signing_name_for_accesspoint_endpoint(self, request):
         accesspoint_service = request.context['s3_accesspoint']['service']
         self._override_signing_name(request.context, accesspoint_service)
@@ -1706,7 +1779,7 @@ class S3EndpointSetter(object):
         original_components = urlsplit(request.url)
         accesspoint_endpoint = urlunsplit((
             original_components.scheme,
-            self._get_accesspoint_netloc(request.context, region_name),
+            self._get_netloc(request.context, region_name),
             self._get_accesspoint_path(
                 original_components.path, request.context),
             original_components.query,
@@ -1716,6 +1789,32 @@ class S3EndpointSetter(object):
             'Updating URI from %s to %s' % (request.url, accesspoint_endpoint))
         request.url = accesspoint_endpoint
 
+    def _get_netloc(self, request_context, region_name):
+        if is_global_accesspoint(request_context):
+            return self._get_mrap_netloc(request_context)
+        else:
+            return self._get_accesspoint_netloc(request_context, region_name)
+
+    def _get_mrap_netloc(self, request_context):
+        s3_accesspoint = request_context['s3_accesspoint']
+        region_name = 's3-global'
+        mrap_netloc_components = [
+            s3_accesspoint['name']
+        ]
+        if self._endpoint_url:
+            endpoint_url_netloc = urlsplit(self._endpoint_url).netloc
+            mrap_netloc_components.append(endpoint_url_netloc)
+        else:
+            partition = s3_accesspoint['partition']
+            mrap_netloc_components.extend(
+                [
+                    'accesspoint',
+                    region_name,
+                    self._get_partition_dns_suffix(partition)
+                ]
+            )
+        return '.'.join(mrap_netloc_components)
+
     def _get_accesspoint_netloc(self, request_context, region_name):
         s3_accesspoint = request_context['s3_accesspoint']
         accesspoint_netloc_components = [
@@ -1750,7 +1849,7 @@ class S3EndpointSetter(object):
         return '.'.join(accesspoint_netloc_components)
 
     def _inject_fips_if_needed(self, component, request_context):
-        if 'fips' in request_context.get('client_region', ''):
+        if self._use_fips_endpoint:
             return '%s-fips' % component
         return component
 
@@ -1763,6 +1862,14 @@ class S3EndpointSetter(object):
         # All S3 operations require at least a / in their path.
         return original_path.replace('/' + name, '', 1) or '/'
 
+    def _get_partition_dns_suffix(self, partition_name):
+        dns_suffix = self._endpoint_resolver.get_partition_dns_suffix(
+            partition_name
+        )
+        if dns_suffix is None:
+            dns_suffix = self._DEFAULT_DNS_SUFFIX
+        return dns_suffix
+
     def _get_dns_suffix(self, region_name):
         resolved = self._endpoint_resolver.construct_endpoint(
             's3', region_name)
@@ -1859,17 +1966,18 @@ class S3EndpointSetter(object):
         return fix_s3_host
 
 
-
 class S3ControlEndpointSetter(object):
     _DEFAULT_PARTITION = 'aws'
     _DEFAULT_DNS_SUFFIX = 'amazonaws.com'
     _HOST_LABEL_REGEX = re.compile(r'^[a-zA-Z0-9\-]{1,63}$')
 
     def __init__(self, endpoint_resolver, region=None,
-                 s3_config=None, endpoint_url=None, partition=None):
+                 s3_config=None, endpoint_url=None, partition=None,
+                 use_fips_endpoint=False):
         self._endpoint_resolver = endpoint_resolver
         self._region = region
         self._s3_config = s3_config
+        self._use_fips_endpoint = use_fips_endpoint
         if s3_config is None:
             self._s3_config = {}
         self._endpoint_url = endpoint_url
@@ -1889,7 +1997,6 @@ class S3ControlEndpointSetter(object):
             self._add_headers_from_arn_details(request)
         elif self._use_endpoint_from_outpost_id(request):
             self._validate_outpost_redirection_valid(request)
-            outpost_id = request.context['outpost_id']
             self._override_signing_name(request, 's3-outposts')
             new_netloc = self._construct_outpost_endpoint(self._region)
             self._update_request_netloc(request, new_netloc)
@@ -1901,6 +2008,13 @@ class S3ControlEndpointSetter(object):
         return 'outpost_id' in request.context
 
     def _validate_endpoint_from_arn_details_supported(self, request):
+        if 'fips' in request.context['arn_details']['region']:
+            raise UnsupportedS3ControlArnError(
+                arn=request.context['arn_details']['original'],
+                msg={
+                    'Invalid ARN, FIPS region not allowed in ARN.'
+                }
+            )
         if not self._s3_config.get('use_arn_region', False):
             arn_region = request.context['arn_details']['region']
             if arn_region != self._region:
@@ -2008,11 +2122,16 @@ class S3ControlEndpointSetter(object):
                 region_name,
                 self._get_dns_suffix(region_name),
             ]
+            self._add_fips(netloc)
         return self._construct_netloc(netloc)
 
     def _construct_netloc(self, netloc):
         return '.'.join(netloc)
 
+    def _add_fips(self, netloc):
+        if self._use_fips_endpoint:
+            netloc[0] = netloc[0] + '-fips'
+
     def _add_dualstack(self, netloc):
         if self._s3_config.get('use_dualstack_endpoint'):
             netloc.append('dualstack')
